//=========================================================
// LPC1114 Project
//=========================================================
// File Name : http.c
// Function  : HTTP Server
//---------------------------------------------------------
// Rev.01 2012.03.31 Munetomo Maruyama
//---------------------------------------------------------
// Copyright (C) 2010-2012 Munetomo Maruyama
//=========================================================
// ---- License Information -------------------------------
// Anyone can FREELY use this code fully or partially
// under conditions shown below.
// 1. You may use this code only for individual purpose,
//    and educational purpose.
//    Do not use this code for business even if partially.
// 2. You can copy, modify and distribute this code.
// 3. You should remain this header text in your codes
//   including Copyright credit and License Information.
// 4. Your codes should inherit this license information.
//=========================================================
// ---- Patent Notice -------------------------------------
// I have not cared whether this system (hw + sw) causes
// infringement on the patent, copyright, trademark,
// or trade secret rights of others. You have all
// responsibilities for determining if your designs
// and products infringe on the intellectual property
// rights of others, when you use technical information
// included in this system for your business.
//=========================================================
// ---- Disclaimers ---------------------------------------
// The function and reliability of this system are not
// guaranteed. They may cause any damages to loss of
// properties, data, money, profits, life, or business.
// By adopting this system even partially, you assume
// all responsibility for its use.
//=========================================================

#ifdef __USE_CMSIS
#include "LPC11xx.h"
#endif

#include <string.h>
#include "adc.h"
#include "color_led.h"
#include "http.h"
#include "uart.h"
#include "utility.h"

#include "../jpeg/marykiban.h" // JPEG Data

//=================
// Define Macro
//=================
#define HH(x)  (((x) >> 24) & 0x00ff)
#define HL(x)  (((x) >> 16) & 0x00ff)
#define LH(x)  (((x) >>  8) & 0x00ff)
#define LL(x)  (((x) >>  0) & 0x00ff)
#define MSB(x) (((x) >>  8) & 0x00ff)
#define LSB(x) (((x) >>  0) & 0x00ff)

//==================================
// Define POST Message Length (max)
//==================================
#define POST_STRING_MAX 32

//================================
// Define Web Page
//================================
// [Note]
// A Check Box in OFF state will not return any data in response message.
// So, it was not used in this sample application.
//
static const uint8_t HEADER_HTML[] =
{
        "HTTP/1.1 200 OK\r\n"
        "Cache-control: no-cache\r\n"
        "Connection: close\r\n"
        "Content-Length: ##LEN_BODYHTML##\r\n"
        "Content-Type: text/html\r\n"
        "\r\n"
};
static const uint8_t HEADER_JPEG[] =
{
        "HTTP/1.1 200 OK\r\n"
        "Cache-control: no-cache\r\n"
        "Connection: close\r\n"
        "Content-Length: ##LEN_JPEGDATA##\r\n"
        "Content-Type: image/jpeg\r\n"
        "\r\n"
};
static const uint8_t HEADER_NOCONTENT[] =
{
        "HTTP/1.1 204 No Content\r\n"
        "Cache-control: no-cache\r\n"
        "Connection: close\r\n"
        "\r\n"
};
static const uint8_t BODY_INDEX_HTML[] =
{
        "<!DOCTYPE html>\r\n"
        "<HTML>\r\n"
        "<HEAD>\r\n"
        "<TITLE>MARY Web Server</TITLE>\r\n"
        "</HEAD>\r\n"
        "<BODY>\r\n"

        "<SCRIPT TYPE=\"text/javascript\">\r\n"
        "<!--\r\n"

        "function LoadFrame()\r\n"
        "{\r\n"
        "    var frm = document.getElementById('FRM_ADCDATA');\r\n"
        "    frm.src = 'frame.html';\r\n"
        "    setTimeout(\"LoadFrame()\", 1000);\r\n"
        "}\r\n"
        "function LoadImage()\r\n"
        "{\r\n"
        "    var img = document.getElementById(\"IMG_MARYKIBAN\");\r\n"
        "    img.src = \"marykiban.jpeg\";\r\n"
        "    setTimeout(\"LoadFrame()\", 500);\r\n"
        "}\r\n"
        "window.onload = function()\r\n"
        "{\r\n"
        "    setTimeout(\"LoadImage()\", 500);\r\n"
        "}\r\n"

        "// -->\r\n"
        "</SCRIPT>\r\n"

        "<H2>MARY Web Server</H2>\r\n"
        "<IMG ID=\"IMG_MARYKIBAN\" WIDTH=\"128\" HEIGHT=\"128\" ALT=\"\">\r\n"

        "<P>\r\n"
        "<FORM ACTION=\"/index.html\" METHOD=\"POST\">\r\n"
        "LED RED \r\n"
        "<INPUT TYPE=\"RADIO\" NAME=\"RED\" VALUE=\"0\" #CHECK__RED_OFF#>OFF\r\n"
        "<INPUT TYPE=\"RADIO\" NAME=\"RED\" VALUE=\"1\" #CHECK__RED__ON#>ON \r\n"
        "<BR>\r\n"
        "LED GRN \r\n"
        "<INPUT TYPE=\"RADIO\" NAME=\"GRN\" VALUE=\"0\" #CHECK__GRN_OFF#>OFF\r\n"
        "<INPUT TYPE=\"RADIO\" NAME=\"GRN\" VALUE=\"1\" #CHECK__GRN__ON#>ON \r\n"
        "<BR>\r\n"
        "LED BLU \r\n"
        "<INPUT TYPE=\"RADIO\" NAME=\"BLU\" VALUE=\"0\" #CHECK__BLU_OFF#>OFF\r\n"
        "<INPUT TYPE=\"RADIO\" NAME=\"BLU\" VALUE=\"1\" #CHECK__BLU__ON#>ON \r\n"
        "<BR>\r\n"
        "<INPUT TYPE=\"SUBMIT\" VALUE=\"UPDATE\">\r\n"
        "</FORM>\r\n"
        "</P>\r\n"

        "<P>\r\n"
        "Realtime ADC Data\r\n"
        "<BR>\r\n"
        "<IFRAME ID=\"FRM_ADCDATA\" NAME=\"FRM_ADCDATA\" WIDTH=\"220\" HEIGHT=\"35\" SCROLLING=\"no\">\r\n"
        "You need a browser which supports 'IFRAME' tag.\r\n"
        "</IFRAME>\r\n"
        "</P>\r\n"

        "<NOSCRIPT>\r\n"
        "You need a browser which supports JavaScript.\r\n"
        "</NOSCRIPT>\r\n"

        "</BODY>\r\n"
        "</HTML>\r\n"
};
static const uint8_t BODY_FRAME_HTML[] =
{
        "<!DOCTYPE html>\r\n"
        "<HTML>\r\n"
        "<HEAD>\r\n"
        "<TITLE>Auto Update Information</TITLE>\r\n"
        "</HEAD>\r\n"
        "<BODY onLoad=\"draw_adc_graph()\">\r\n"

        "<SCRIPT type=\"text/javascript\">\r\n"
        "<!--\r\n"

        "function draw_adc_graph()\r\n"
        "{\r\n"
        "    var adcdata;\r\n"
        "    adcdata = ####ADC_DATA####;\r\n"

        "    var canvas = document.getElementById('adc_graph');\r\n"
        "    if (canvas.getContext)\r\n"
        "    {\r\n"
        "        var context = canvas.getContext('2d');\r\n"
        "        context.fillText(\"\"+adcdata, 5, 13);\r\n"

        "        context.strokeStyle = 'rgb(000,255,000)'; // GRN\r\n"
        "        context.strokeRect(30,5,160,10);\r\n"

        "        context.strokeStyle = 'rgb(000,255,000)'; // GRN\r\n"
        "        context.fillStyle   = 'rgb(000,255,000)'; // GRN\r\n"
        "        context.fillRect(30,5,(adcdata*160/1024),10);\r\n"
        "    }\r\n"
        "}\r\n"

        "// -->\r\n"
        "</SCRIPT>\r\n"

        "<CANVAS ID=\"adc_graph\" WIDTH=\"200\" HEIGHT=\"30\">\r\n"
        "You need a browser which supports 'CANVAS' tag.\r\n"
        "</CANVAS>\r\n"
        "</BODY>\r\n"
        "</HTML>\r\n"
};

//=====================================
// Define Structure for Search String
//=====================================
typedef struct
{
    uint8_t *str;
    uint32_t str_pos;
    uint32_t str_len;
} sSearchString;

//================
// Define State
//================
enum HTTP_Server_STATE
{
    INIT,
    IDLE,
    GET_FOUND,
    GET_INDEX_FOUND,
    GET_FRAME_FOUND,
    GET_JPEG_FOUND,
    POST_FOUND,
    POST_CONTENT_LENGTH,
    POST_CONTENT_STRING,
    OUTPUT_INDEX_PAGE,
    OUTPUT_FRAME_PAGE,
    OUTPUT_JPEG,
    OUTPUT_NOCONTENT
};

//=============
// Globals
//=============
uint32_t gStatus_LED_RED;
uint32_t gStatus_LED_GRN;
uint32_t gStatus_LED_BLU;

//===========================
// Initialize Search String
//===========================
void Init_Search_String(char *str, sSearchString *pSS)
{
    pSS->str = (uint8_t*)str;
    pSS->str_pos = 0;
    pSS->str_len = strlen((char*)str);
}

//====================
// Search String
//====================
uint32_t Search_String(uint8_t ch, sSearchString *pSS)
{
    uint32_t result;

    result = 0;
    if (ch == pSS->str[pSS->str_pos])
    {
        pSS->str_pos++;
        if (pSS->str_pos >= pSS->str_len)
        {
            result = 1;
            pSS->str_pos = 0;
        }
    }
    else
    {
        pSS->str_pos = 0;
    }
    return result;
}

//=====================
// Replace String
//=====================
// Replace a string with 16 length to new one with same length.
// If new one is shorter than old one, spaces will be padded.
void Replace_String(uint8_t *buf, uint8_t *str_old, uint8_t *str_new)
{
    uint8_t *pos;
    uint32_t i, end;

    pos = (uint8_t*) strstr((char*)buf, (char*)str_old);
    if (pos == NULL) return;
    //
    end = 0;
    for (i = 0; i < 16; i++)
    {
        if (str_new[i] == '\0') end = 1;
        if (end)
            pos[i] = ' ';
        else
            pos[i] = str_new[i];
    }
}

//===============================
// HTTP Server
//===============================
void HTTP_Server(uint8_t ch, uint32_t ip, uint32_t port)
{
    static uint32_t state = INIT;
    static sSearchString sSS_GET;
    static sSearchString sSS_POST;
    static sSearchString sSS_DEFAULT_PAGE;
    static sSearchString sSS_INDEX_HTML;
    static sSearchString sSS_FRAME_HTML;
    static sSearchString sSS_MARYKIBAN_JPG;
    static sSearchString sSS_CONTENT_LENGTH;
    static sSearchString sSS_CRLFCRLF;
    static uint8_t STR_CHECK_ON[]  = "checked         ";
    static uint8_t STR_CHECK_OFF[] = "                ";
    static uint8_t BUF_SEND[4096];
    static uint8_t  post_string[POST_STRING_MAX];
    static uint32_t post_string_pos;
    static uint32_t post_length;
    static uint32_t post_length_finish;
    uint32_t len_head;
    uint32_t len_body;
    uint8_t  buf_temp[16];
    uint32_t buf_len = 16;
    //------------------
    // Initialization
    //------------------
    if (state == INIT)
    {
        Init_Search_String("GET",  &sSS_GET);
        Init_Search_String("POST", &sSS_POST);
        Init_Search_String("/ ",   &sSS_DEFAULT_PAGE);
        Init_Search_String("/index.html",     &sSS_INDEX_HTML);
        Init_Search_String("/frame.html",     &sSS_FRAME_HTML);
        Init_Search_String("/marykiban.jpeg", &sSS_MARYKIBAN_JPG);
        Init_Search_String("Content-Length: ", &sSS_CONTENT_LENGTH);
        Init_Search_String("\r\n\r\n", &sSS_CRLFCRLF);
        state = IDLE;
    }
    //--------------------
    // Search Keywords
    //--------------------
    switch(state)
    {
        //----------------------------------
        case IDLE :
        {
            if (Search_String(ch, &sSS_POST))
            {
                state = POST_FOUND;
            }
            if (Search_String(ch, &sSS_GET))
            {
                state = GET_FOUND;
            }
            break;
        }
        //----------------------------------
        case GET_FOUND :
        {
            if (Search_String(ch, &sSS_DEFAULT_PAGE))
            {
                state = GET_INDEX_FOUND;
            }
            if (Search_String(ch, &sSS_INDEX_HTML))
            {
                state = GET_INDEX_FOUND;
            }
            if (Search_String(ch, &sSS_FRAME_HTML))
            {
                state = GET_FRAME_FOUND;
            }
            if (Search_String(ch, &sSS_MARYKIBAN_JPG))
            {
                state = GET_JPEG_FOUND;
            }
            if (Search_String(ch, &sSS_CRLFCRLF))
            {
                state = OUTPUT_NOCONTENT;
            }
            break;
        }
        //----------------------------------
        case GET_INDEX_FOUND :
        {
            if (Search_String(ch, &sSS_CRLFCRLF))
            {
                state = OUTPUT_INDEX_PAGE;
            }
            break;
        }
        //----------------------------------
        case GET_FRAME_FOUND :
        {
            if (Search_String(ch, &sSS_CRLFCRLF))
            {
                state = OUTPUT_FRAME_PAGE;
            }
            break;
        }
        //----------------------------------
        case GET_JPEG_FOUND :
        {
            if (Search_String(ch, &sSS_CRLFCRLF))
            {
                state = OUTPUT_JPEG;
            }
            break;
        }
        //----------------------------------
        case POST_FOUND :
        {
            if (Search_String(ch, &sSS_CONTENT_LENGTH))
            {
                state = POST_CONTENT_LENGTH;
                post_length = 0;
                post_length_finish = 0;
            }
            break;
        }
        //----------------------------------
        case POST_CONTENT_LENGTH :
        {
            if ((post_length_finish == 0) && (ch >= '0') && (ch <= '9'))
            {
                post_length = post_length * 10 + (ch - '0');
            }
            else
            {
                post_length_finish = 1;
                //
                if (Search_String(ch, &sSS_CRLFCRLF))
                {
                    if (post_length > 0)
                    {
                        state = POST_CONTENT_STRING;
                        post_string_pos = 0;
                    }
                    else
                    {
                        state = IDLE;
                    }
                }
            }
            break;
        }
        //----------------------------------
        case POST_CONTENT_STRING :
        {
            // (example) RED=0&GRN=0&BLU=0
            post_string[post_string_pos++] = ch;
            post_length--;
            //
            if (post_length == 0)
            {
                gStatus_LED_RED = (post_string[ 4] == '1')? 1 : 0;
                gStatus_LED_GRN = (post_string[10] == '1')? 1 : 0;
                gStatus_LED_BLU = (post_string[16] == '1')? 1 : 0;
                state = OUTPUT_INDEX_PAGE;
            }
            else if (post_string_pos >= POST_STRING_MAX)
            {
                state = OUTPUT_INDEX_PAGE;
            }
            break;
        }
        //----------------------------------
        default :
        {
            state = IDLE; // illegal error
            break;
        }
    }
    //------------------------
    // Output Index Page
    //------------------------
    if (state == OUTPUT_INDEX_PAGE)
    {
        // Take each Length
        len_head = strlen((char*)HEADER_HTML);
        len_body = strlen((char*)BODY_INDEX_HTML);
        //
        // HTML Header
        //
        // Copy and Edit HTML Header
        memcpy(BUF_SEND, HEADER_HTML, len_head);
        BUF_SEND[len_head] = '\0';
        xitoa(buf_temp, &buf_len, len_body, 10, 0);
        Replace_String(BUF_SEND, (uint8_t*)"##LEN_BODYHTML##", buf_temp);
        // Output HTML Header
        XBee_Put_Tx_Packet(len_head, BUF_SEND, ip, port);
        //
        // HTML Body
        //
        // Copy and Edit HTML Body
        memcpy(BUF_SEND, BODY_INDEX_HTML, len_body);
        BUF_SEND[len_body] = '\0';
        if (gStatus_LED_RED)
        {
            Replace_String(BUF_SEND, (uint8_t*)"#CHECK__RED_OFF#", STR_CHECK_OFF);
            Replace_String(BUF_SEND, (uint8_t*)"#CHECK__RED__ON#", STR_CHECK_ON);
        }
        else
        {
            Replace_String(BUF_SEND, (uint8_t*)"#CHECK__RED_OFF#", STR_CHECK_ON);
            Replace_String(BUF_SEND, (uint8_t*)"#CHECK__RED__ON#", STR_CHECK_OFF);
        }
        if (gStatus_LED_GRN)
        {
            Replace_String(BUF_SEND, (uint8_t*)"#CHECK__GRN_OFF#", STR_CHECK_OFF);
            Replace_String(BUF_SEND, (uint8_t*)"#CHECK__GRN__ON#", STR_CHECK_ON);
        }
        else
        {
            Replace_String(BUF_SEND, (uint8_t*)"#CHECK__GRN_OFF#", STR_CHECK_ON);
            Replace_String(BUF_SEND, (uint8_t*)"#CHECK__GRN__ON#", STR_CHECK_OFF);
        }
        if (gStatus_LED_BLU)
        {
            Replace_String(BUF_SEND, (uint8_t*)"#CHECK__BLU_OFF#", STR_CHECK_OFF);
            Replace_String(BUF_SEND, (uint8_t*)"#CHECK__BLU__ON#", STR_CHECK_ON);
        }
        else
        {
            Replace_String(BUF_SEND, (uint8_t*)"#CHECK__BLU_OFF#", STR_CHECK_ON);
            Replace_String(BUF_SEND, (uint8_t*)"#CHECK__BLU__ON#", STR_CHECK_OFF);
        }
        // Output HTML Page
        XBee_Put_Tx_Packet(len_body, BUF_SEND, ip, port);
        //
        state = IDLE;
    }
    //------------------------
    // Output Frame Page
    //------------------------
    if (state == OUTPUT_FRAME_PAGE)
    {
        // Take each Length
        len_head = strlen((char*)HEADER_HTML);
        len_body = strlen((char*)BODY_FRAME_HTML);
        //
        // HTML Header
        //
        // Copy and Edit HTML Header
        memcpy(BUF_SEND, HEADER_HTML, len_head);
        BUF_SEND[len_head] = '\0';
        xitoa(buf_temp, &buf_len, len_body, 10, 0);
        Replace_String(BUF_SEND, (uint8_t*)"##LEN_BODYHTML##", buf_temp);
        // Output HTML Header
        XBee_Put_Tx_Packet(len_head, BUF_SEND, ip, port);
        //
        // HTML Body
        //
        // Copy and Edit HTML Body
        memcpy(BUF_SEND, BODY_FRAME_HTML, len_body);
        BUF_SEND[len_body] = '\0';
        xitoa(buf_temp, &buf_len, Get_ADC_Data(0), 10, 0);
        Replace_String(BUF_SEND, (uint8_t*)"####ADC_DATA####", buf_temp);
        // Output HTML Page
        XBee_Put_Tx_Packet(len_body, BUF_SEND, ip, port);
        //
        state = IDLE;
    }
    //------------------------
    // Output JPEG
    //------------------------
    if (state == OUTPUT_JPEG)
    {
        // Take Length
        len_head = strlen((char*)HEADER_JPEG);
        //
        // JPEG Header
        //
        // Copy and Edit JPEG Header
        memcpy(BUF_SEND, HEADER_JPEG, len_head);
        BUF_SEND[len_head] = '\0';
        xitoa(buf_temp, &buf_len, marykiban_jpg_size, 10, 0);
        Replace_String(BUF_SEND, (uint8_t*)"##LEN_JPEGDATA##", buf_temp);
        // Output JPEG Header
        XBee_Put_Tx_Packet(len_head, BUF_SEND, ip, port);
        //
        // JPEG Data
        //
        XBee_Put_Tx_Packet(marykiban_jpg_size, (uint8_t*)marykiban_jpg, ip, port);
        //
        state = IDLE;
    }
    //-------------------------
    // Output No Content
    //-------------------------
    if (state == OUTPUT_NOCONTENT)
    {
        // Take each Length
        len_head = strlen((char*)HEADER_NOCONTENT);
        //
        // HTML Header
        //
        // Copy and Edit HTML Header
        memcpy(BUF_SEND, HEADER_NOCONTENT, len_head);
        BUF_SEND[len_head] = '\0';
        // Output HTML Header
        XBee_Put_Tx_Packet(len_head, BUF_SEND, ip, port);
        //
        state = IDLE;
    }
}

//====================================
// UART Send Byte with Escape Control
//====================================
void UARTSend_Byte_Escape(uint8_t ch)
{
    if ((ch == 0x7e) || (ch == 0x7d) || (ch == 0x11) || (ch == 0x13))
    {
        UARTSend_Byte(0x7d);
        UARTSend_Byte(ch ^ 0x20);
    }
    else
    {
        UARTSend_Byte(ch);
    }
}

//===============================
// Put XBee TX Packet (IPV4)
//===============================
void XBee_Put_Tx_Packet(uint32_t length, uint8_t *buf, uint32_t ip, uint32_t port)
{
    uint32_t len_packet;
    uint32_t len_remain;
    uint32_t buf_position;
    uint32_t checksum;
    uint32_t i;

    len_remain = length;
    buf_position = 0;
    //-------------------------
    // Send Multiple Packets
    //-------------------------
    while(1)
    {
        //---------------
        // Send Header
        //---------------
        UARTSend_Byte(0x7e); // Delimiter
        len_packet = (len_remain > 1400)? 1400 : len_remain;
        len_remain = len_remain - len_packet;
        UARTSend_Byte_Escape(MSB(len_packet + 12)); // Length MSB
        UARTSend_Byte_Escape(LSB(len_packet + 12)); // Length LSB
        UARTSend_Byte_Escape(0x20); // ID for Tx IPv4
        UARTSend_Byte_Escape(0x01); // Enable Tx Status Frame
        UARTSend_Byte_Escape(HH(ip)); // Destination IP Address HH
        UARTSend_Byte_Escape(HL(ip)); // Destination IP Address HL
        UARTSend_Byte_Escape(LH(ip)); // Destination IP Address LH
        UARTSend_Byte_Escape(LL(ip)); // Destination IP Address LL
        UARTSend_Byte_Escape(MSB(port)); // Destination Port MSB
        UARTSend_Byte_Escape(LSB(port)); // Destination Port LSB
        UARTSend_Byte_Escape(0x00); // Source Port MSB
        UARTSend_Byte_Escape(0x50); // Source Port LSB
        UARTSend_Byte_Escape(0x01); // Protocol TCP
        UARTSend_Byte_Escape(0x00); // Leave TCP Socket Open
        checksum = HH(ip) + HL(ip) + LH(ip) + LL(ip)
                 + MSB(port) + LSB(port)
                 + 0x20+0x01+0x50+0x01;
        //---------------
        // Send Body
        //---------------
        for (i = 0; i < len_packet; i++)
        {
            UARTSend_Byte_Escape(buf[buf_position + i]); // RF Data
            checksum = checksum + buf[buf_position + i];
        }
        buf_position = buf_position + len_packet;
        //-----------------
        // Send Checksum
        //-----------------
        checksum = (0xff - checksum) & 0xff;
        UARTSend_Byte_Escape((uint8_t) checksum);
        //---------
        // Finish?
        //---------
        if (len_remain == 0) break;
    }
}

//===============================
// Get XBee RX Packet (IPV4)
//===============================
enum XBee_API_STATE
{
    XBEEAPI_RX_DELIMITER, // 1byte
    XBEEAPI_RX_LEN,       // 2bytes
    XBEEAPI_RX_ID,        // 1byte
    XBEEAPI_RX_SKIP,      // LEN bytes
    XBEEAPI_RX_IP_SRC,    // 4bytes
    XBEEAPI_RX_PORT_DST,  // 2bytes
    XBEEAPI_RX_PORT_SRC,  // 2bytes
    XBEEAPI_RX_PROTOCOL,  // 1byte
    XBEEAPI_RX_STATUS,    // 1byte
    XBEEAPI_RX_RFDATA,    // (LEN - 11) bytes
    XBEEAPI_RX_TXSTATUS,  // 2bytes
    XBEEAPI_RX_CHECKSUM   // 1byte
};
void XBee_Get_Rx_Packet(uint8_t ch)
{
    static uint32_t state = XBEEAPI_RX_DELIMITER;
    static uint32_t bytes = 1;
    static uint32_t escape = 0;
    static uint32_t checksum;
    static uint32_t len;
    static uint32_t ip_src;
    static uint32_t port_dst;
    static uint32_t port_src;
    static uint32_t protocol;
    static uint32_t txframeid;
    static uint32_t txstatus;
    //-----------------
    // Escaped?
    //-----------------
    if (escape == 1)
    {
        ch = ch ^ 0x20;
        escape = 0;
    }
    else
    {
        if (ch == 0x7d) // escape code
        {
            escape = 1;
            return; // skip this code
        }
    }
    //--------------------
    // Decode the Packet
    //--------------------
    switch(state)
    {
        //------------------------------------------
        case XBEEAPI_RX_DELIMITER:
        {
            state     = (ch == 0x7e)? XBEEAPI_RX_LEN : state;
            bytes     = (ch == 0x7e)? 2 : bytes;
            checksum  = 0;
            len       = 0;
            ip_src    = 0;
            port_dst  = 0;
            port_src  = 0;
            protocol  = 0;
            txframeid = 0;
            txstatus  = 0;
            break;
        }
        //------------------------------------------
        case XBEEAPI_RX_LEN:
        {
            bytes = bytes - 1;
            len   = len | ((uint32_t)ch << (8 * bytes));
            state = (bytes == 0)? XBEEAPI_RX_ID : state;
            bytes = (bytes == 0)? 1 : bytes;
            break;
        }
        //------------------------------------------
        case XBEEAPI_RX_ID:
        {
            checksum = checksum + ch;
            state  = (ch == 0xb0)? XBEEAPI_RX_IP_SRC :
                     (ch == 0x89)? XBEEAPI_RX_TXSTATUS :
                                   XBEEAPI_RX_SKIP;
            bytes  = (ch == 0xb0)? 4 :
                     (ch == 0x89)? 2 :
                                   len;
            break;
        }
        //------------------------------------------
        case XBEEAPI_RX_IP_SRC:
        {
            checksum = checksum + ch;
            bytes = bytes - 1;
            ip_src = ip_src | ((uint32_t)ch << (8 * bytes));
            state  = (bytes == 0)? XBEEAPI_RX_PORT_DST : state;
            bytes  = (bytes == 0)? 2 : bytes;
            break;
        }
        //------------------------------------------
        case XBEEAPI_RX_PORT_DST:
        {
            checksum = checksum + ch;
            bytes = bytes - 1;
            port_dst = port_dst | ((uint32_t)ch << (8 * bytes));
            state  = (bytes == 0)? XBEEAPI_RX_PORT_SRC : state;
            bytes  = (bytes == 0)? 2 : bytes;
            break;
        }
        //------------------------------------------
        case XBEEAPI_RX_PORT_SRC:
        {
            checksum = checksum + ch;
            bytes = bytes - 1;
            port_src = port_src | ((uint32_t)ch << (8 * bytes));
            state  = (bytes == 0)? XBEEAPI_RX_PROTOCOL : state;
            bytes  = (bytes == 0)? 1 : bytes;
            break;
        }
        //------------------------------------------
        case XBEEAPI_RX_PROTOCOL:
        {
            checksum = checksum + ch;
            protocol = ch;
            state  = XBEEAPI_RX_STATUS;
            bytes  = 1;
            break;
        }
        //------------------------------------------
        case XBEEAPI_RX_STATUS:
        {
            checksum = checksum + ch;
            state  = XBEEAPI_RX_RFDATA;
            bytes  = len - 11;
            break;
        }
        //------------------------------------------
        case XBEEAPI_RX_RFDATA:
        {
            checksum = checksum + ch;
            bytes = bytes - 1;
            state  = (bytes == 0)? XBEEAPI_RX_CHECKSUM : state;
            bytes  = (bytes == 0)? 1 : bytes;
            //
            HTTP_Server(ch, ip_src, port_src);
            //
            break;
        }
        //------------------------------------------
        case XBEEAPI_RX_SKIP:
        {
            checksum = checksum + ch;
            bytes = bytes - 1;
            state  = (bytes == 0)? XBEEAPI_RX_CHECKSUM : state;
            bytes  = (bytes == 0)? 1 : bytes;
            break;
        }
        //------------------------------------------
        case XBEEAPI_RX_TXSTATUS:
        {
            checksum = checksum + ch;
            bytes = bytes - 1;
            txframeid = (bytes == 1)? ch : txframeid;
            txstatus  = (bytes == 0)? ch : txstatus;
            state  = (bytes == 0)? XBEEAPI_RX_CHECKSUM : state;
            bytes  = (bytes == 0)? 1 : bytes;
            break;
        }
        //------------------------------------------
        case XBEEAPI_RX_CHECKSUM:
        {
            checksum = (0xff - checksum) & 0xff;
            state  = XBEEAPI_RX_DELIMITER;
            bytes  = 1;
            break;
        }
        //------------------------------------------
        default : // Illegal
        {
            state = XBEEAPI_RX_DELIMITER;
            break;
        }
    }
}


//===============================
// Main HTTP Server
//===============================
void Main_HTTP_Server(void)
{
    uint8_t  ch;
    //----------------------------
    // Initialize Local Hardware
    //----------------------------
    Init_ADC();
    UARTInit(230400);
    //
    gStatus_LED_RED = 0;
    gStatus_LED_GRN = 0;
    gStatus_LED_BLU = 0;
    //---------------------
    // Main Loop
    //---------------------
    while(1)
    {
        //------------------------
        // Parse UART Data
        //------------------------
        if (UARTReceive_Check())
        {
            ch = UARTReceive_Byte();
            XBee_Get_Rx_Packet(ch);
        }
        //------------------------
        // Set LED
        //------------------------
        Set_RED_LED(gStatus_LED_RED);
        Set_GRN_LED(gStatus_LED_GRN);
        Set_BLU_LED(gStatus_LED_BLU);
    }
}

//=========================================================
// End of Program
//=========================================================
